@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2020 Zabbix SIA
+** Copyright (C) 2001-2022 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -25,18 +25,24 @@
 
 require_once dirname(__FILE__).'/js/configuration.triggers.edit.js.php';
 
-$widget = (new CWidget())->setTitle(_('Triggers'));
+$widget = (new CWidget())
+	->setTitle(_('Triggers'))
+	->setDocUrl(CDocHelper::getUrl(CDocHelper::CONFIGURATION_TRIGGERS_EDIT));
 
 // Append host summary to widget header.
 if ($data['hostid'] != 0) {
-	$widget->addItem(get_header_host_table('triggers', $data['hostid']));
+	$widget->setNavigation(getHostNavigation('triggers', $data['hostid']));
 }
 
+$url = (new CUrl('triggers.php'))
+	->setArgument('context', $data['context'])
+	->getUrl();
+
 // Create form.
-$triggersForm = (new CForm())
+$triggersForm = (new CForm('post', $url))
 	->setid('triggers-form')
 	->setName('triggersForm')
-	->setAttribute('aria-labeledby', ZBX_STYLE_PAGE_TITLE)
+	->setAttribute('aria-labelledby', ZBX_STYLE_PAGE_TITLE)
 	->addVar('form', $data['form'])
 	->addVar('hostid', $data['hostid'])
 	->addVar('expression_constructor', $data['expression_constructor'])
@@ -44,7 +50,8 @@ $triggersForm = (new CForm())
 	->addVar('toggle_expression_constructor', '')
 	->addVar('toggle_recovery_expression_constructor', '')
 	->addVar('remove_expression', '')
-	->addVar('remove_recovery_expression', '');
+	->addVar('remove_recovery_expression', '')
+	->addVar('backurl', $data['backurl']);
 
 $discovered_trigger = false;
 
@@ -79,6 +86,7 @@ if ($discovered_trigger) {
 			->setArgument('form', 'update')
 			->setArgument('parent_discoveryid', $data['discoveryRule']['itemid'])
 			->setArgument('triggerid', $data['triggerDiscovery']['parent_triggerid'])
+			->setArgument('context', $data['context'])
 	));
 }
 
@@ -104,10 +112,10 @@ $triggersFormList
 
 if ($discovered_trigger) {
 	$triggersFormList->addVar('priority', (int) $data['priority']);
-	$severity = new CSeverity(['name' => 'priority_names', 'value' => (int) $data['priority']], false);
+	$severity = new CSeverity('priority_names', (int) $data['priority'], false);
 }
 else {
-	$severity = new CSeverity(['name' => 'priority', 'value' => (int) $data['priority']]);
+	$severity = new CSeverity('priority', (int) $data['priority']);
 }
 
 $triggersFormList->addRow(_('Severity'), $severity);
@@ -121,15 +129,14 @@ if ($data['recovery_expression_field_readonly']) {
 	$triggersForm->addItem((new CVar('recovery_expression', $data['recovery_expression']))->removeId());
 }
 
-$popup_options = [
-	'srctbl' => $data['expression_field_name'],
-	'srcfld1' => $data['expression_field_name'],
+$popup_parameters = [
 	'dstfrm' => $triggersForm->getName(),
-	'dstfld1' => $data['expression_field_name']
+	'dstfld1' => $data['expression_field_name'],
+	'context' => $data['context']
 ];
 
 if ($data['hostid']) {
-	$popup_options['hostid'] = $data['hostid'];
+	$popup_parameters['hostid'] = $data['hostid'];
 }
 
 $expression_row = [
@@ -142,11 +149,15 @@ $expression_row = [
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-	(new CButton('insert', ($data['expression_constructor'] == IM_TREE) ? _('Edit') : _('Add')))
+	(new CButton('insert', $data['expression_constructor'] == IM_TREE ? _('Edit') : _('Add')))
 		->addClass(ZBX_STYLE_BTN_GREY)
-		->onClick('return PopUp("popup.triggerexpr",jQuery.extend('.json_encode($popup_options).
-			',{expression: jQuery(\'[name="'.$data['expression_field_name'].'"]\').val()}), null, this);'
-		)
+		->setAttribute('data-parameters', json_encode($popup_parameters))
+		->onClick('
+			PopUp("popup.triggerexpr", {
+				...JSON.parse(this.dataset.parameters),
+				expression: document.querySelector("[name='.$data['expression_field_name'].']").value
+			}, {dialogue_class: "modal-popup-generic"});
+		')
 		->setEnabled(!$readonly)
 		->removeId()
 ];
@@ -163,28 +174,28 @@ if ($data['expression_constructor'] == IM_TREE) {
 	if ($data['expression_formula'] === '') {
 		// Append "Add" button.
 		$expression_row[] = (new CSimpleButton(_('Add')))
-			->onClick('javascript: submitFormWithParam("'.$triggersForm->getName().'", "add_expression", "1");')
+			->onClick('submitFormWithParam("'.$triggersForm->getName().'", "add_expression", "1");')
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->setEnabled(!$readonly);
 	}
 	else {
 		// Append "And" button.
 		$expression_row[] = (new CSimpleButton(_('And')))
-			->onClick('javascript: submitFormWithParam("'.$triggersForm->getName().'", "and_expression", "1");')
+			->onClick('submitFormWithParam("'.$triggersForm->getName().'", "and_expression", "1");')
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->setEnabled(!$readonly);
 
 		// Append "Or" button.
 		$expression_row[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 		$expression_row[] = (new CSimpleButton(_('Or')))
-			->onClick('javascript: submitFormWithParam("'.$triggersForm->getName().'", "or_expression", "1");')
+			->onClick('submitFormWithParam("'.$triggersForm->getName().'", "or_expression", "1");')
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->setEnabled(!$readonly);
 
 		// Append "Replace" button.
 		$expression_row[] = (new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN);
 		$expression_row[] = (new CSimpleButton(_('Replace')))
-			->onClick('javascript: submitFormWithParam("'.$triggersForm->getName().'", "replace_expression", "1");')
+			->onClick('submitFormWithParam("'.$triggersForm->getName().'", "replace_expression", "1");')
 			->addClass(ZBX_STYLE_BTN_GREY)
 			->setEnabled(!$readonly);
 	}
@@ -192,7 +203,7 @@ if ($data['expression_constructor'] == IM_TREE) {
 elseif ($data['expression_constructor'] != IM_FORCED) {
 	$input_method_toggle = (new CSimpleButton(_('Expression constructor')))
 		->addClass(ZBX_STYLE_BTN_LINK)
-		->onClick('javascript: '.
+		->onClick(
 			'document.getElementById("toggle_expression_constructor").value=1;'.
 			'document.getElementById("expression_constructor").value='.
 				(($data['expression_constructor'] == IM_TREE) ? IM_ESTABLISHED : IM_TREE).';'.
@@ -261,12 +272,13 @@ if ($data['expression_constructor'] == IM_TREE) {
 						? (new CCol(
 							(new CSimpleButton(_('Remove')))
 								->addClass(ZBX_STYLE_BTN_LINK)
-								->onClick('javascript:'.
-									' if (confirm('.json_encode(_('Delete expression?')).')) {'.
-										' delete_expression("'.$e['id'] .'", '.TRIGGER_EXPRESSION.');'.
-										' document.forms["'.$triggersForm->getName().'"].submit();'.
-									' }'
-								)
+								->setAttribute('data-id', $e['id'])
+								->onClick('
+									if (confirm('.json_encode(_('Delete expression?')).')) {
+										delete_expression(this.dataset.id, '.TRIGGER_EXPRESSION.');
+										document.forms["'.$triggersForm->getName().'"].submit();
+									}
+								')
 						))->addClass(ZBX_STYLE_NOWRAP)
 						: null,
 					makeInformationList($info_icons)
@@ -280,8 +292,11 @@ if ($data['expression_constructor'] == IM_TREE) {
 	}
 
 	$testButton = (new CButton('test_expression', _('Test')))
-		->onClick('return PopUp("popup.testtriggerexpr",{expression: this.form.elements["expression"].value}, null,'.
-					'this);')
+		->onClick(
+			'return PopUp("popup.testtriggerexpr", {expression: this.form.elements["expression"].value}, {
+				dialogue_class: "modal-popup-generic"
+			});'
+		)
 		->addClass(ZBX_STYLE_BTN_LINK)
 		->removeId();
 
@@ -321,15 +336,14 @@ $triggersFormList->addRow(_('OK event generation'),
 		->setEnabled(!$readonly)
 );
 
-$popup_options = [
-	'srctbl' => $data['recovery_expression_field_name'],
-	'srcfld1' => $data['recovery_expression_field_name'],
+$popup_parameters = [
 	'dstfrm' => $triggersForm->getName(),
-	'dstfld1' => $data['recovery_expression_field_name']
+	'dstfld1' => $data['recovery_expression_field_name'],
+	'context' => $data['context']
 ];
 
 if ($data['hostid']) {
-	$popup_options['hostid'] = $data['hostid'];
+	$popup_parameters['hostid'] = $data['hostid'];
 }
 
 $recovery_expression_row = [
@@ -342,12 +356,15 @@ $recovery_expression_row = [
 		->setWidth(ZBX_TEXTAREA_STANDARD_WIDTH)
 		->setAriaRequired(),
 	(new CDiv())->addClass(ZBX_STYLE_FORM_INPUT_MARGIN),
-	(new CButton('insert', ($data['recovery_expression_constructor'] == IM_TREE) ? _('Edit') : _('Add')))
+	(new CButton('insert', $data['recovery_expression_constructor'] == IM_TREE ? _('Edit') : _('Add')))
 		->addClass(ZBX_STYLE_BTN_GREY)
-		->onClick('return PopUp("popup.triggerexpr",jQuery.extend('.
-			json_encode($popup_options).
-				',{expression: jQuery(\'[name="'.$data['recovery_expression_field_name'].'"]\').val()}), null, this);'
-		)
+		->setAttribute('data-parameters', json_encode($popup_parameters))
+		->onClick('
+			PopUp("popup.triggerexpr", {
+				...JSON.parse(this.dataset.parameters),
+				expression: document.querySelector("[name='.$data['recovery_expression_field_name'].']").value
+			}, {dialogue_class: "modal-popup-generic"});
+		')
 		->setEnabled(!$readonly)
 		->removeId()
 ];
@@ -459,12 +476,13 @@ if ($data['recovery_expression_constructor'] == IM_TREE) {
 						? (new CCol(
 							(new CSimpleButton(_('Remove')))
 								->addClass(ZBX_STYLE_BTN_LINK)
-								->onClick('javascript:'.
-									' if (confirm('.json_encode(_('Delete expression?')).')) {'.
-										' delete_expression("'.$e['id'] .'", '.TRIGGER_RECOVERY_EXPRESSION.');'.
-										' document.forms["'.$triggersForm->getName().'"].submit();'.
-									' }'
-								)
+								->setAttribute('data-id', $e['id'])
+								->onClick('
+									if (confirm('.json_encode(_('Delete expression?')).')) {
+										delete_expression(this.dataset.id, '.TRIGGER_RECOVERY_EXPRESSION.');
+										document.forms["'.$triggersForm->getName().'"].submit();
+									}
+								')
 						))->addClass(ZBX_STYLE_NOWRAP)
 						: null,
 					makeInformationList($info_icons)
@@ -478,8 +496,11 @@ if ($data['recovery_expression_constructor'] == IM_TREE) {
 	}
 
 	$testButton = (new CButton('test_expression', _('Test')))
-		->onClick('return PopUp("popup.testtriggerexpr",'.
-			'{expression: this.form.elements["recovery_expression"].value}, null, this);')
+		->onClick(
+			'return PopUp("popup.testtriggerexpr", {expression: this.form.elements["recovery_expression"].value}, {
+				dialogue_class: "modal-popup-generic"
+			});'
+		)
 		->addClass(ZBX_STYLE_BTN_LINK)
 		->removeId();
 
@@ -570,7 +591,9 @@ $triggersTab->addTab('tags-tab', _('Tags'), new CPartial('configuration.tags.tab
 		'source' => 'trigger',
 		'tags' => $data['tags'],
 		'show_inherited_tags' => $data['show_inherited_tags'],
-		'readonly' => $discovered_trigger
+		'readonly' => $discovered_trigger,
+		'tabs_id' => 'tabs',
+		'tags_tab_id' => 'tags-tab'
 	]),
 	TAB_INDICATOR_TAGS
 );
@@ -593,13 +616,18 @@ foreach ($data['db_dependencies'] as $dependency) {
 
 	$dependenciesTable->addRow(
 		(new CRow([
-			(new CLink($dep_trigger_description, 'triggers.php?form=update&triggerid='.$dependency['triggerid']))
-				->setAttribute('target', '_blank'),
+			(new CLink($dep_trigger_description,
+				(new CUrl('triggers.php'))
+					->setArgument('form', 'update')
+					->setArgument('triggerid', $dependency['triggerid'])
+					->setArgument('context', $data['context'])
+			))->setTarget('_blank'),
 			(new CCol(
 				$discovered_trigger
 					? null
 					: (new CButton('remove', _('Remove')))
-						->onClick('javascript: removeDependency("'.$dependency['triggerid'].'");')
+						->setAttribute('data-triggerid', $dependency['triggerid'])
+						->onClick('view.removeDependency(this.dataset.triggerid)')
 						->addClass(ZBX_STYLE_BTN_LINK)
 						->removeId()
 			))->addClass(ZBX_STYLE_NOWRAP)
@@ -607,45 +635,82 @@ foreach ($data['db_dependencies'] as $dependency) {
 	);
 }
 
+$buttons = null;
+
+if (!$discovered_trigger) {
+	$buttons = $data['context'] === 'host'
+		? (new CButton('add_dep_trigger', _('Add')))
+			->setAttribute('data-hostid', $data['hostid'])
+			->onClick('
+				PopUp("popup.generic", {
+					srctbl: "triggers",
+					srcfld1: "triggerid",
+					reference: "deptrigger",
+					hostid: this.dataset.hostid,
+					multiselect: 1,
+					with_triggers: 1,
+					real_hosts: 1
+				}, {dialogue_class: "modal-popup-generic"});
+			')
+			->addClass(ZBX_STYLE_BTN_LINK)
+		: new CHorList([
+				(new CButton('add_dep_trigger', _('Add')))
+					->setAttribute('data-templateid', $data['hostid'])
+					->onClick('
+						PopUp("popup.generic", {
+							srctbl: "template_triggers",
+							srcfld1: "triggerid",
+							reference: "deptrigger",
+							templateid: this.dataset.templateid,
+							multiselect: 1,
+							with_triggers: 1
+						}, {dialogue_class: "modal-popup-generic"});
+					')
+					->addClass(ZBX_STYLE_BTN_LINK),
+				(new CButton('add_dep_host_trigger', _('Add host trigger')))
+					->onClick('
+						PopUp("popup.generic", {
+							srctbl: "triggers",
+							srcfld1: "triggerid",
+							reference: "deptrigger",
+							multiselect: 1,
+							with_triggers: 1,
+							real_hosts: 1
+						}, {dialogue_class: "modal-popup-generic"});
+					')
+					->addClass(ZBX_STYLE_BTN_LINK)
+		]);
+}
+
 $dependenciesFormList->addRow(_('Dependencies'),
-	(new CDiv([
-		$dependenciesTable,
-		$discovered_trigger
-			? null
-			: (new CButton('bnt1', _('Add')))
-				->onClick('return PopUp("popup.generic",'.
-					json_encode([
-						'srctbl' => 'triggers',
-						'srcfld1' => 'triggerid',
-						'reference' => 'deptrigger',
-						'hostid' => $data['hostid'],
-						'multiselect' => '1',
-						'with_triggers' => '1',
-						'noempty' => '1'
-					]).', null, this);'
-				)
-				->addClass(ZBX_STYLE_BTN_LINK)
-	]))
+	(new CDiv([$dependenciesTable, $buttons]))
 		->addClass(ZBX_STYLE_TABLE_FORMS_SEPARATOR)
 		->addStyle('min-width: '.ZBX_TEXTAREA_BIG_WIDTH.'px;')
 );
 $triggersTab->addTab('dependenciesTab', _('Dependencies'), $dependenciesFormList, TAB_INDICATOR_DEPENDENCY);
 
+$cancelButton = $data['backurl'] !== null
+	? (new CRedirectButton(_('Cancel'), $data['backurl']))->setId('cancel')
+	: new CButtonCancel(url_param('context'));
+
 // Append buttons to form list.
 if (!empty($data['triggerid'])) {
 	$triggersTab->setFooter(makeFormFooter(
 		new CSubmit('update', _('Update')), [
 			new CSubmit('clone', _('Clone')),
-			(new CButtonDelete(_('Delete trigger?'), url_params(['form', 'hostid', 'triggerid'])))
-				->setEnabled(!$data['limited']),
-			new CButtonCancel()
+			(new CButtonDelete(
+				_('Delete trigger?'),
+				url_params(['form', 'hostid', 'triggerid', 'context', 'backurl']),
+				'context'
+			))->setEnabled(!$data['limited']),
+			$cancelButton
 		]
 	));
 }
 else {
 	$triggersTab->setFooter(makeFormFooter(
 		new CSubmit('add', _('Add')),
-		[new CButtonCancel()]
+		[$cancelButton]
 	));
 }
 
@@ -655,3 +720,11 @@ $triggersForm->addItem($triggersTab);
 $widget->addItem($triggersForm);
 
 $widget->show();
+
+(new CScriptTag('
+	view.init('.json_encode([
+		'form_name' => $triggersForm->getName()
+	]).');
+'))
+	->setOnDocumentReady()
+	->show();
